import 'dart:convert';
import 'package:flutter/services.dart';
import 'package:nui_auth/domain/nui_user_activity.dart';
import 'package:nui_auth/domain/nui_user_info.dart';
import 'package:nui_core/data/nui_data_util.dart';
import '../nui_auth.dart';

abstract class IAuthBridgeInterface{
  Future<dynamic> signUp({required MethodCall call});
  Future<dynamic> verifyEmail({required MethodCall call});
  Future<dynamic> resendEmailVerificationCode({required MethodCall call});
  Future<dynamic> login({required MethodCall call});
  Future<dynamic> loginWithGoogle({required MethodCall call});
  Future<dynamic> loginWithApple({required MethodCall call});
  Future<dynamic> renewAccessToken({required MethodCall call});
  Future<dynamic> changePassword({required MethodCall call});
  Future<dynamic> forgotPassword({required MethodCall call});
  Future<dynamic> getAccessToken({required MethodCall call});
  Future<dynamic> getRefreshToken({required MethodCall call});
  Future<dynamic> getUserId({required MethodCall call});
  Future<dynamic> logout({required MethodCall call});
  Future<dynamic> isLoggedIn({required MethodCall call});
  Future<dynamic> isTokenExpired({required MethodCall call});
  Future<dynamic> logUserActivity({required MethodCall call});
  Future<dynamic> updateUserInfo({required MethodCall call});
  Future<dynamic> updateLastActive({required MethodCall call});
}

class NUIAuthBridge with IAuthBridgeInterface{
  static const METHOD_CHANNEL = "com.inglab.iauthmod/auth";
  static const platform = const MethodChannel(METHOD_CHANNEL);

  static const CHANNEL_INIT = "initialize";
  static const CHANNEL_SIGN_UP = "signUp";
  static const CHANNEL_VERIFY_EMAIL = "verifyEmail";
  static const CHANNEL_RESEND_EMAIL_VERIFICATION_CODE = "resendEmailVerificationCode";
  static const CHANNEL_LOGIN = "login";
  static const CHANNEL_LOGIN_WITH_GOOGLE = "loginWithGoogle";
  static const CHANNEL_LOGIN_WITH_APPLE = "loginWithApple";
  static const CHANNEL_RENEW_ACCESS_TOKEN = "renewAccessToken";
  static const CHANNEL_CHANGE_PASSWORD = "changePassword";
  static const CHANNEL_FORGOT_PASSWORD = "forgotPassword";
  static const CHANNEL_GET_ACCESS_TOKEN = "getAccessToken";
  static const CHANNEL_GET_REFRESH_TOKEN = "getRefreshToken";
  static const CHANNEL_GET_USER_ID = "getUserId";
  static const CHANNEL_LOGOUT = "logout";
  static const CHANNEL_IS_LOGGED_IN = "isLoggedIn";
  static const CHANNEL_IS_TOKEN_EXPIRED = "isTokenExpired";
  static const CHANNEL_LOG_USER_ACTIVITY = "logUserActivity";
  static const CHANNEL_UPDATE_USER_INFO = "updateUserInfo";
  static const CHANNEL_UPDATE_LAST_ACTIVE = "updateLastActive";

  void initialize(){
    platform.setMethodCallHandler((call) async{
      dynamic result;
      try{
        switch(call.method){
          case CHANNEL_INIT: result = _initializeModule(call: call); break;
          case CHANNEL_SIGN_UP: result = signUp(call: call); break;
          case CHANNEL_VERIFY_EMAIL: result = verifyEmail(call: call); break;
          case CHANNEL_RESEND_EMAIL_VERIFICATION_CODE: result = resendEmailVerificationCode(call: call); break;
          case CHANNEL_LOGIN: result = login(call: call); break;
          case CHANNEL_LOGIN_WITH_GOOGLE: result = loginWithGoogle(call: call); break;
          case CHANNEL_LOGIN_WITH_APPLE: result = loginWithApple(call: call); break;
          case CHANNEL_RENEW_ACCESS_TOKEN: result = renewAccessToken(call: call); break;
          case CHANNEL_CHANGE_PASSWORD: result = changePassword(call: call); break;
          case CHANNEL_FORGOT_PASSWORD: result = forgotPassword(call: call); break;
          case CHANNEL_GET_ACCESS_TOKEN: result = getAccessToken(call: call); break;
          case CHANNEL_GET_REFRESH_TOKEN: result = getRefreshToken(call: call); break;
          case CHANNEL_GET_USER_ID: result = getUserId(call: call); break;
          case CHANNEL_LOGOUT: result = logout(call: call); break;
          case CHANNEL_IS_LOGGED_IN: result = isLoggedIn(call: call); break;
          case CHANNEL_IS_TOKEN_EXPIRED: result = isTokenExpired(call: call); break;
          case CHANNEL_LOG_USER_ACTIVITY: result = logUserActivity(call: call); break;
          case CHANNEL_UPDATE_USER_INFO: result = updateUserInfo(call: call); break;
          case CHANNEL_UPDATE_LAST_ACTIVE: result = updateLastActive(call: call); break;
          default: return null;
        }
        platform.invokeMethod(call.method, jsonEncode(await result));
      }catch(e, s){
        platform.invokeMethod(call.method, jsonEncode({
          "success": false,
          "message": "Failed to process request with error : $e"
        }));
      }
      return true;
    });
  }

  NUIAuth _getModuleInstance() => NUIAuth.get();

  Future _initializeModule({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final baseUrl = payload["baseUrl"];
    final appVersion = payload["appVersion"];
    await NUIAuthBuilder()
        .appVersion(appVersion)
        .baseUrl(baseUrl)
        .build();

    return {
      "success": true,
      "message": "IAuth initialized",
    };
  }

  @override
  Future changePassword({required MethodCall call}) async {
    final payload = jsonDecode(call.arguments);
    final oldPassword = payload["oldPassword"];
    final newPassword = payload["newPassword"];
    final confirmPassword = payload["confirmPassword"];
    final result = await _getModuleInstance().changePassword(oldPassword: oldPassword, newPassword: newPassword, confirmPassword: confirmPassword);

    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future forgotPassword({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final email = payload["email"];
    final result = await _getModuleInstance().forgotPassword(email: email);

    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future getAccessToken({required MethodCall call}) async{
    final accessToken = await _getModuleInstance().getAccessToken();
    return {
      "success": accessToken != null,
      "message": accessToken != null ? "Get access token successful" : "No access token found",
      "data": {
        "token": accessToken?.token,
        "refreshToken": accessToken?.refreshToken,
        "claims": accessToken?.claims,
        "payload": accessToken?.payload
      }
    };
  }

  @override
  Future getRefreshToken({required MethodCall call}) async{
    final refreshToken = _getModuleInstance().getRefreshToken();
    return {
      "success": !isNullOrEmpty(refreshToken),
      "message": !isNullOrEmpty(refreshToken) ? "Get refresh token successful" : "No refresh token found",
      "data": {
        "refreshToken": refreshToken
      }
    };
  }

  @override
  Future getUserId({required MethodCall call}) async{
    final userId = _getModuleInstance().getUserId();
    return {
      "success": !isNullOrEmpty(userId),
      "message": !isNullOrEmpty(userId) ? "Get user ID successful" : "No user ID found",
      "data": {
        "userId": userId
      }
    };
  }

  @override
  Future isLoggedIn({required MethodCall call}) async{
    final isLoggedIn = await _getModuleInstance().isLoggedIn();
    return {
      "success": true,
      "message": isLoggedIn ? "User is logged in" : "User is not logged in",
      "data": {
        "isLoggedIn": isLoggedIn
      }
    };
  }

  @override
  Future isTokenExpired({required MethodCall call}) async{
    final isTokenExpired = await _getModuleInstance().isTokenExpired();
    return {
      "success": true,
      "message": isTokenExpired ? "Token has expired" : "Token has not expired",
      "data": {
        "isTokenExpired": isTokenExpired
      }
    };
  }

  @override
  Future logUserActivity({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final activity = NUIUserActivityEntity().toEntity(payload);

    final result = await _getModuleInstance().logUserActivity(userActivity: activity);
    return {
      "success": result.success,
      "message": result.message
    };
  }

  @override
  Future login({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    String username = payload["username"];
    String password = payload["password"];

    final result = await _getModuleInstance().login(username: username, password: password);
    return {
      "success": result.success,
      "message": result.message
    };
  }

  @override
  Future loginWithApple({required MethodCall call}) async{
    return {
      "success": false,
      "message": "Login with Apple is not available yet",
    };
  }

  @override
  Future loginWithGoogle({required MethodCall call}) async{
    final result = await _getModuleInstance().loginWithGoogle();
    return {
      "success": false,
      "message": "Login with Google is not available yet",
    };
  }

  @override
  Future logout({required MethodCall call}) async{
    final result = await _getModuleInstance().logout();
    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future renewAccessToken({required MethodCall call}) async{
    // final result = await _getModuleInstance().renewAccessToken();
    final result = await _getModuleInstance().renewToken();
    return {
      "success": result.success,
      "message": result.message,
      "data": {
        "accessToken" : result.data?.accessToken,
        "refreshToken": result.data?.refreshToken
      }
    };
  }

  @override
  Future resendEmailVerificationCode({required MethodCall call}) async{
    final result = await _getModuleInstance().resendEmailVerificationCode();
    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future signUp({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final email = payload["email"];
    final username = payload["username"];
    final password = payload["password"];

    final result = await _getModuleInstance().signup(username: username, email: email, password: password);
    return {
      "success": result.success,
      "message": result.message,
      "data": {
        // "signUpToken": result.data?.signupToken
      }
    };
  }

  @override
  Future updateLastActive({required MethodCall call}) async{
    final result = await _getModuleInstance().updateLastActive();
    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future updateUserInfo({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final userInfo = NUIUserInfoEnt().toEntity(payload);

    final result = await _getModuleInstance().updateUserInfo(userInfo: userInfo);
    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future verifyEmail({required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final verificationCode = payload["code"];

    final result = await _getModuleInstance().verifyEmail(code: verificationCode);
    return {
      "success": result.success,
      "message": result.message,
    };
  }

}