library nui_auth;
import 'package:flutter/foundation.dart';

import 'package:flutter/cupertino.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:nui_auth/domain/nui_user_activity.dart';
import 'package:nui_auth/result/nui_apple_login_result.dart';
import 'package:nui_auth/web/json/req/nui_auth_change_password.dart';
import 'package:nui_auth/web/json/req/nui_auth_device_info.dart';
import 'package:nui_auth/web/json/req/nui_auth_forgot_password.dart';
import 'package:nui_auth/web/json/req/nui_auth_login.dart';
import 'package:nui_auth/web/json/req/nui_auth_renew_token.dart';
import 'package:nui_auth/web/json/req/nui_auth_signup.dart';
import 'package:nui_core/code/nui_device_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_geolocator/nui_geolocator.dart';
import 'package:nui_web/nui_web.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';
import 'domain/nui_auth_token.dart';
import 'domain/nui_user_info.dart';
import 'web/nui_auth_web_json.dart';
import 'web/nui_auth_api.dart';
import 'result/nui_google_login_result.dart';
import 'result/nui_auth_result.dart';

typedef NUITokenExpiredListener = void Function();
typedef HeadersBuilder = Map<String, String> Function();

abstract class NUIAuthInterface{

  Future<void> saveToken({required String accessToken, required String refreshToken, int? expiresIn, String provider});
  Future<void> clearToken();
  Future<NUIAuthAccessToken?> getAccessToken();
  Future<String?> getRefreshToken();
  Future<String?> getUserId();
  Future<bool> isLoggedIn();
  Future<bool> isTokenExpired();
  Future clearRememberedAccount();
  Future rememberEmail({required String email, required String password});
  Future<AuthEmailAcc?> getRememberedEmail();
  Future<DataResult<bool>> signup({required String username, required String password, required String email, String? mobileNo, String? name, String? displayName, int? age, String? gender, String? recoveryEmail, String? provider});
  Future<DataResult<NUIAuthLoginResp>> login({required String username, required String password, String? provider});
  Future<DataResult<NUIAuthLoginResp>> renewToken();
  Future<DataResult<bool>> changePassword({required String oldPassword, required String newPassword, required String confirmPassword, String? provider});
  Future<DataResult<bool>> forgotPassword({required String email, String? provider});
  Future<DataResult<bool>> logout({String? provider});
  Future<DataResult<NUIAuthLoginResp>> adminLogin({required String username, required String password, String? provider});
  Future<DataResult<NUIAppleLoginResult>> loginWithApple(BuildContext context);
  Future<DataResult> logoutFromGoogle(BuildContext context);
  Future<DataResult<NUIGoogleLoginResult>> loginWithGoogle();

  //TODO: Yet to implement, currently iAuth v3 doesn't have api below
  Future<DataResult> verifyEmail({required String code});
  Future<DataResult> resendEmailVerificationCode();
  Future<DataResult> logUserActivity({required NUIUserActivity userActivity});
  Future<DataResult> updateUserInfo({required NUIUserInfo userInfo});
  Future<DataResult> updateLastActive();

}

class NUIAuthMessages{
  NUIAuthMessages._builder();
  static const String TOKEN_REFRESHED = "Token is refreshed successfully";
  static const String TOKEN_EXPIRED = "Token has expired";
  static const String TOKEN_VALID = "Token still valid";
  static const String FAILED_TO_REFRESH_TOKEN_LOGIN = "Your session has expired, please login again";
  static const String INVALID_SIGNUP_TOKEN = "Invalid sign up token found";
  static const String GOOGLE_LOGOUT_SUCCESS = "Logout from Google successful";
  static const String GOOGLE_LOGOUT_FAILED = "Failed to logout from Google";
  static const String GOOGLE_LOGIN_SUCCESS = "Google login successful";
  static const String GOOGLE_LOGIN_UNSUCCESSFUL = "Login with Google was unsuccessful";
  static const String GOOGLE_LOGIN_FAILED = "Failed to login with Google";

  static const String APPLE_LOGOUT_SUCCESS = "Logout from Apple successful";
  static const String APPLE_LOGOUT_FAILED = "Failed to logout from Apple";
  static const String APPLE_LOGIN_SUCCESS = "Apple login successful";
  static const String APPLE_LOGIN_UNSUCCESSFUL = "Login with Apple was unsuccessful";
  static const String APPLE_LOGIN_FAILED = "Failed to login with Apple";
}

class NUIAuth implements NUIAuthInterface{
  static const String GRANT_TYPE_PASSWORD = "password";
  static const String GRANT_TYPE_REFRESH_TOKEN = "refresh_token";
  static const String _PREF_ACCESS_TOKEN = "istudio_access_token";
  static const String _PREF_REFRESH_TOKEN = "istudio_refresh_token";
  static const String _PREF_EXPIRES_IN = "istudio_expires_in";
  static const String _PREF_SSO_ACCESS_TOKEN = "istudio_sso_access_token";
  static const String _PREF_SIGNUP_TOKEN = "istudio_signup_token";
  static const String _PREF_APP_INSTANCE_ID = "istudio_app_instance_id";
  static const String _MODULE = "NUIAuth";

  FlutterSecureStorage secureStorage = FlutterSecureStorage();
  late String _baseUrl;
  late NUIAuthApi _api;
  late GoogleSignIn _googleSignIn;
  late String _appVersion;
  late String? _provider;
  NUIWebBuilder? _webBuilder;
  HeadersBuilder? _headers;
  late bool _saveTokenWithSecureStorage;
  static NUIAuth? _instance ;
  NUIAuth._builder(NUIAuthBuilder builder){
    _baseUrl = builder._baseUrl;
    _api = NUIAuthApi();
    _googleSignIn = GoogleSignIn();
    _provider = builder._provider;
    _appVersion = builder._appVersion;
    _webBuilder = builder._webBuilder;
    _headers = builder._headers;
    _saveTokenWithSecureStorage = builder._saveTokenWithSecureStorage ?? true;
    _buildNUIWeb();
  }

  Future<Map<String, String>> _defaultHeaders() async {
    NUIDeviceInfo? deviceInfo;
    Map<String, String> headers = {};
    if(defaultTargetPlatform == TargetPlatform.iOS || defaultTargetPlatform == TargetPlatform.android) {
      deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    }
    NUIAuthAccessToken? accessToken = await NUIAuth.get().getAccessToken();
    headers = {
      "DeviceId": deviceInfo?.uuid ?? "",
      "Content-Type": "application/json",
      "X-DeviceOs": deviceInfo?.platform ?? "",
      "X-DeviceOs-Version": deviceInfo?.version ?? "",
      "Device": deviceInfo?.model ?? "",
      "Version": _appVersion
    };
    if(!isNullOrEmpty(accessToken)){
      headers["Authorization"] = "Bearer ${accessToken?.token ?? "no-token"}";
    }

    if(_headers != null){
      final fixedHeaders = _headers!();
      for(String key in fixedHeaders.keys){
        headers[key] = fixedHeaders[key]!;
      }
    }

    return headers;
  }

  void _buildNUIWeb() async{

    final authHeaders = await _defaultHeaders();

    NUIWebBuilder builder;
    if(isNullOrEmpty(_webBuilder)){
      builder = NUIWebBuilder()
          .module(NUIAuthApi.MODULE)
          .baseURL(_baseUrl)
          .headers(() => authHeaders)
          .addInterceptor(NUIAuthWebInterceptor());
    }else{
      _webBuilder!.module(NUIAuthApi.MODULE);
      _webBuilder!.baseURL(_baseUrl);
      _webBuilder!.headers(() => authHeaders);
      builder = _webBuilder!;
    }

    builder.build();
  }

  static NUIAuth get(){
    if(_instance == null) throwNUI(_MODULE, "Instance has not been created with builder");
    return _instance!;
  }

  static String getAccessTokenPref(String pref, {String? provider}){
    var prefix = provider != null ? provider + "_" : "";
    return prefix + pref;
  }

  Future<void> _saveLoginResponseData(NUIAuthAccessToken token, {String? provider, String? origin}) async{
    logNUI("NUIAuth", "Saving login response, accessToken: ${token.token}, refreshToken: ${token.refreshToken}");
    if(_saveTokenWithSecureStorage){
      await secureStorage.write(key: getAccessTokenPref(_PREF_ACCESS_TOKEN, provider: provider), value: token.token);
      await secureStorage.write(key: getAccessTokenPref(_PREF_REFRESH_TOKEN, provider: provider), value: token.refreshToken);
    }else{
      await NUISharedPref.get().save(getAccessTokenPref(_PREF_ACCESS_TOKEN, provider: provider), token.token);
      await NUISharedPref.get().save(getAccessTokenPref(_PREF_REFRESH_TOKEN, provider: provider), token.refreshToken);
    }
  }

  Future<NUIAuthDeviceInfo> _getDeviceInfo() async {
    final deviceMeta = await NUIDeviceUtil.getDeviceInfo();
    return NUIAuthDeviceInfo(
        deviceId: deviceMeta?.uuid ?? "",
        deviceOs: deviceMeta?.platform ?? "",
        deviceModel: deviceMeta?.model ?? "",
        osVersion: deviceMeta?.version ?? "",
        ipAddress: deviceMeta?.ipAddress ?? ""
    );
  }

  Future<void> _setRememberedEmail(String email) async {
    await secureStorage.write(key: "auth_remembered_email", value: email);
  }

  Future<void> _setRememberedPassword(String password) async{
    await secureStorage.write(key: "auth_remembered_password", value: password);
  }

  Future<String?> _getRememberedEmail() async {
    return secureStorage.read(key: 'auth_remembered_email');
  }

  Future<String?> _getRememberedPassword() async {
    return secureStorage.read(key: 'auth_remembered_password');
  }

  @override
  Future clearRememberedAccount() async {
    await secureStorage.delete(key: 'auth_remembered_email');
    await secureStorage.delete(key: 'auth_remembered_password');
    return true;
  }

  @override
  Future rememberEmail({required String email, required String password}) async{
    if(isNullOrEmpty(email) || isNullOrEmpty(password)) return null;
    await _setRememberedEmail(email);
    await _setRememberedPassword(password);
    return;
  }

  @override
  Future<AuthEmailAcc?> getRememberedEmail() async{
    final email = await _getRememberedEmail();
    final password = await _getRememberedPassword();
    if(isNullOrEmpty(email) || isNullOrEmpty(password)) return null;
    return AuthEmailAcc(email: email!, password: AuthPassword(password!));
  }

  @override
  Future<void> saveToken({required String accessToken,required String refreshToken, int? expiresIn, String? provider}) async {
    await NUISharedPref.get().save(getAccessTokenPref(_PREF_EXPIRES_IN, provider: provider), expiresIn);
    if(_saveTokenWithSecureStorage){
      await secureStorage.write(key: getAccessTokenPref(_PREF_ACCESS_TOKEN, provider: provider), value: accessToken);
      await secureStorage.write(key: getAccessTokenPref(_PREF_REFRESH_TOKEN, provider: provider), value: refreshToken);
    }else{
      await NUISharedPref.get().save(getAccessTokenPref(_PREF_ACCESS_TOKEN, provider: provider), accessToken);
      await NUISharedPref.get().save(getAccessTokenPref(_PREF_REFRESH_TOKEN, provider: provider), refreshToken);
    }
  }

  Future<void> _deleteToken() async {
    await NUISharedPref.get().remove(getAccessTokenPref(_PREF_ACCESS_TOKEN));
    await NUISharedPref.get().remove(getAccessTokenPref(_PREF_REFRESH_TOKEN));
  }

  Future<void> _deleteSecureToken() async {
    await secureStorage.delete(key: getAccessTokenPref(_PREF_ACCESS_TOKEN));
    await secureStorage.delete(key: getAccessTokenPref(_PREF_REFRESH_TOKEN));
  }

  @override
  Future<void> clearToken() async {
    _saveTokenWithSecureStorage ? await _deleteSecureToken() : _deleteToken();
  }

  Future<String?> _getAccessToken({String? provider})async{
    return _saveTokenWithSecureStorage ? await secureStorage.read(key: getAccessTokenPref(_PREF_ACCESS_TOKEN, provider: provider)) : NUISharedPref.get().quickGetString(getAccessTokenPref(_PREF_ACCESS_TOKEN, provider: provider));
  }

  Future<String?> _getRefreshToken() async {
    return _saveTokenWithSecureStorage ? await secureStorage.read(key: _PREF_REFRESH_TOKEN) : NUISharedPref.get().quickGetString(_PREF_REFRESH_TOKEN);
  }

  @override
  Future<NUIAuthAccessToken?> getAccessToken({String? provider})async{
    final accessToken = await _getAccessToken();
    final refreshToken = await getRefreshToken();
    if(!isNullOrEmpty(accessToken)) {
      return NUIAuthAccessToken(token: accessToken!, refreshToken: refreshToken ?? "");
    }
    return null;
  }

  @override
  Future<String?> getRefreshToken() async {
    return _getRefreshToken();
  }

  @override
  Future<String?> getUserId({String? provider}) async {
    final token = await getAccessToken(provider: provider);
    return token?.getUserId();
  }

  @override
  Future<bool> isLoggedIn({String? provider}) async {
    final accessToken = await getAccessToken(provider: provider);
    return accessToken != null;
  }

  @override
  Future<bool> isTokenExpired({String? provider}) async {
    final accessToken = await getAccessToken(provider: provider);
    if(accessToken == null) return false;
    return accessToken.isExpired() != true;
  }

  @override
  Future<DataResult<bool>> signup({required String username, required String password, required String email, String? mobileNo, String? name, String? displayName, int? age, String? gender, String? recoveryEmail, String? provider}) async{
    final req = NUIAuthSignUpReq(username: username, password: password, email: email, name: name, displayName: displayName, age: age, gender: gender, mobileNo: mobileNo, recoveryEmail: recoveryEmail, provider: provider);
    final result = await _api.signup(request: req);
    return result;
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> login({required String username, required String password, String? provider}) async{
    final req = NUIAuthLoginReq(
        username: username,
        password: password,
        provider: provider ?? _provider,
        deviceInfo: await _getDeviceInfo()
    );
    logNUI("NUIAuth", "Logging in with username : $username, password : $password");
    final result = await _api.login(request: req);
    if(result.success && result.data != null) {
      final authToken = NUIAuthAccessToken(token: result.data!.accessToken, refreshToken: result.data!.refreshToken);
      await _saveLoginResponseData(authToken);
      logNUI("NUIAuth", "Login successful with userId : ${authToken.getUserId()}");
      return result;
    }
    return result;
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> renewToken({String? provider}) async{
    final refreshToken = await getRefreshToken();
    final req = NUIAuthRenewTokenReq(
        refreshToken: refreshToken ?? "",
        deviceInfo: await _getDeviceInfo()
    );
    logNUI("NUIAuth", "Refreshing token with refresh token : $refreshToken");
    final result = await _api.renewToken(request: req);
    if(result.success && result.data != null) {
      final authToken = NUIAuthAccessToken(token: result.data!.accessToken, refreshToken: result.data!.refreshToken);
      await _saveLoginResponseData(authToken, provider: provider);
      return result;
    }
    else{
      return result;
    }
  }

  @override
  Future<DataResult<bool>> forgotPassword({required String email, String? provider}) async{
    final req = NUIAuthForgotPasswordReq(email: email, provider: provider ?? _provider);
    final result = await _api.forgotPassword(request: req);
    return result;
  }

  @override
  Future<DataResult<bool>> changePassword({required String oldPassword, required String newPassword, required String confirmPassword, String? provider}) async{
    final req = NUIAuthChangePasswordReq(oldPassword: oldPassword, newPassword: newPassword, confirmPassword: confirmPassword, provider: provider ?? _provider);
    final result = await _api.changePassword(request: req);
    return result;
  }

  @override
  Future<DataResult<bool>> logout({String? provider}) async{
    final accessToken = await getAccessToken(provider: provider);
    await clearToken();
    final result = await _api.logout(accessToken: accessToken?.token ?? "");
    return result;
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> adminLogin({required String username, required String password, String? provider}) async{
    final req = NUIAuthLoginReq(
        username: username,
        password: password,
        provider: provider ?? _provider,
        deviceInfo: await _getDeviceInfo()
    );
    logNUI("NUIAuth", "Logging in admin with username : $username, password : $password");
    final result = await _api.login(request: req);
    if(result.success && result.data != null) {
      final authToken = NUIAuthAccessToken(token: result.data!.accessToken, refreshToken: result.data!.refreshToken);
      await _saveLoginResponseData(authToken);
      logNUI("NUIAuth", "Admin login successful with userId : ${authToken.getUserId()}");
      return result;
    }
    return result;
  }

  @override
  Future<DataResult<NUIAppleLoginResult>> loginWithApple(BuildContext context) async{
    try {
      final credential = await SignInWithApple.getAppleIDCredential(
        scopes: [
          AppleIDAuthorizationScopes.email,
          AppleIDAuthorizationScopes.fullName,
        ],
      );

      final bool isSignedIn =  credential.identityToken != null;
      if(isSignedIn) {
        logNUI("NUIAuth", "Apple login successful");

        final appleResp = NUIAppleLoginResult(
            idToken: credential.identityToken,
            email: credential.email,
            familyName: credential.familyName,
            givenName: credential.givenName,
            authCode: credential.authorizationCode,
            state: credential.state,
            userIdentifier: credential.userIdentifier
        );

        // send the token back to iStudio server
        final ssoReq = NUIAuthSSOSignUpReq(provider: NUIAuthSSOSignUpReq.SSO_TYPE_APPLE, providerToken: appleResp.idToken!, email: appleResp.email!);
        final result = await _api.ssoLogin(request: ssoReq);

        if (result.success && result.data != null) {
          // save the 3rd-party access token
          await secureStorage.write(key: _PREF_SSO_ACCESS_TOKEN, value: appleResp.idToken);
          // save iStudio access token
          final authToken = NUIAuthAccessToken(token: result.data!.accessToken, refreshToken: result.data!.refreshToken);
          await _saveLoginResponseData(authToken);
          return DataResult<NUIAppleLoginResult>(success: true, message: NUIAuthMessages.APPLE_LOGIN_SUCCESS, data: appleResp, code: '');
        }
        else {
          return DataResult<NUIAppleLoginResult>(success: false, message: NUIAuthMessages.APPLE_LOGIN_UNSUCCESSFUL, code: '');
        }
      }
      // identity token is null, login with apple unsuccessful
      return DataResult<NUIAppleLoginResult>(success: false, message: NUIAuthMessages.APPLE_LOGIN_UNSUCCESSFUL, code: '');

    } catch(e, s){
      logNUI("NUIAuth", "Sign in with Apple failed with error : $e, stacktrace : $s");
      return DataResult(success: false, message: NUIAuthMessages.APPLE_LOGIN_FAILED, code: '');
    }
  }

  @override
  Future<DataResult<NUIGoogleLoginResult>> loginWithGoogle() async{
    try {
      final loginResult = await _googleSignIn.signIn();

      logNUI("NUIAuth", "Google sign in successful, email : ${loginResult?.email}, name : ${loginResult?.displayName}");
      final bool isSignedIn =  await _googleSignIn.isSignedIn();
      if(isSignedIn){
        final curUser = _googleSignIn.currentUser;

        final authentication = await curUser!.authentication;

        final googleResp = NUIGoogleLoginResult(displayName: curUser.displayName, id: curUser.id, email: curUser.email, photoUrl: curUser.photoUrl, idToken: authentication.idToken, serverAuthCode: curUser.serverAuthCode, isExpired: false);

        // if idToken is null or empty, return google login unsuccessful
        if(authentication.idToken.isNullOrEmpty()){ return DataResult<NUIGoogleLoginResult>(success: false, message: NUIAuthMessages.GOOGLE_LOGIN_UNSUCCESSFUL, code: '');}

        // send the token back to iStudio server
        final ssoReq = NUIAuthSSOSignUpReq(provider: NUIAuthSSOSignUpReq.SSO_TYPE_GOOGLE, providerToken: authentication.idToken!, email: curUser.email);
        final result = await _api.ssoLogin(request: ssoReq);

        if(result.success && result.data != null){
          // save the 3rd-party access token
          await secureStorage.write(key: _PREF_SSO_ACCESS_TOKEN, value:authentication.idToken);
          // save iStudio access token
          final authToken = NUIAuthAccessToken(token: result.data!.accessToken, refreshToken: result.data!.refreshToken);
          await _saveLoginResponseData(authToken);
          return DataResult<NUIGoogleLoginResult>(success: true, message: NUIAuthMessages.GOOGLE_LOGIN_SUCCESS, data: googleResp, code: '');
        }else{
          return DataResult<NUIGoogleLoginResult>(success: false, message: NUIAuthMessages.GOOGLE_LOGIN_UNSUCCESSFUL, code: '');
        }
      }
      else{
        return DataResult<NUIGoogleLoginResult>(success: false, message: NUIAuthMessages.GOOGLE_LOGIN_UNSUCCESSFUL, code: '');
      }
    } catch (error) {
      logNUI(_MODULE, "Failed to login with Google with error : $error");
      return DataResult(success: false, message: NUIAuthMessages.GOOGLE_LOGIN_FAILED, code: '');
    }
  }

  @override
  Future<DataResult> logoutFromGoogle(BuildContext context) async{
    // sign out from google
    try {
      await _googleSignIn.signOut();
      return DataResult(success: true, message: NUIAuthMessages.GOOGLE_LOGOUT_SUCCESS, code: '');
    }catch(e){
      logNUI(_MODULE, "Logout from Google failed with error : $e");
      return DataResult(success: false, message: NUIAuthMessages.GOOGLE_LOGOUT_FAILED, code: '');
    }
  }

  //TODO: Yet to implement, currently iAuth v3 doesn't have api below
  @override
  Future<DataResult> verifyEmail({required String code}) async{
    final token = await NUISharedPref.get().getString(_PREF_SIGNUP_TOKEN);

    if (token.isNullOrEmpty()) {
      throwNUI(_MODULE, "Invalid sign up token found");
    }

    final req = NUIAuthVerifyEmailReq(code: code, token: token);
    final result = await _api.verifyEmail(request: req);

    if(result.success){
      await NUISharedPref.get().remove(_PREF_SIGNUP_TOKEN);
    }
    return DataResult(success: result.success, message: result.message, code: '');
  }

  @override
  Future<DataResult> resendEmailVerificationCode() async {
    final token = await NUISharedPref.get().getString(_PREF_SIGNUP_TOKEN);

    if (token.isNullOrEmpty()) {
      throwNUI(_MODULE, NUIAuthMessages.INVALID_SIGNUP_TOKEN);
      return DataResult(success: false, message: NUIAuthMessages.INVALID_SIGNUP_TOKEN, code: 'c');
    }

    final req = NUIAuthVerifyEmailReq(token: token);
    final result = await _api.resendEmailVerificationCode(request: req);
    return DataResult(success: result.success, message: result.message, code: '');
  }


  @override
  Future<DataResult> logUserActivity({required NUIUserActivity userActivity, String? provider}) async{
    final accessToken = await getAccessToken(provider: provider);
    if(accessToken != null){
      final userId = accessToken.getUserId();
      userActivity.userId = int.parse(userId);
      final result = await _api.logUserActivity(activity: userActivity);
      return DataResult(success: result.success == true || result.status == true, message: result.message, code: result.code.toString());
    }
    return DataResult(success: false, message: "User is not signed in yet for activity logging", code: "");
  }

  @override
  Future<DataResult> updateUserInfo({required NUIUserInfo userInfo, String? provider}) async{
    final accessToken = await getAccessToken(provider: provider);

    if(accessToken == null){
      return DataResult(success: false, message: "User is not signed in yet for activity logging", code: "");
    }

    final hasLocationPermission = await NUIPermissionUtil.hasPermission(Permission.location) && await NUIPermissionUtil.hasPermission(Permission.locationWhenInUse);
    if(hasLocationPermission){
      logNUI("NUIAuth", "Location permission was enabled, attaching location information");
      final geoLocator = await NUIGeoLocatorBuilder().build();
      if(userInfo.areaCode == null || userInfo.latitude == null ||userInfo.longitude == null){
        final latlng = await geoLocator.getCurrentLocation();
        if(latlng != null){
          userInfo.latitude = latlng.latitude;
          userInfo.longitude = latlng.longitude;

          if(userInfo.areaCode == null) {
            final areaCode = await geoLocator.getAreaCodeFromLocation(latlng.latitude, latlng.longitude);
            userInfo.areaCode = areaCode;
          }
        }
      }
    }
    userInfo.userId = int.parse(accessToken.getUserId());
    final result = await _api.updateUserInfo(info: userInfo);
    return DataResult(success: result.success == true || result.status == true, message: result.message, code: '');
  }

  @override
  Future<DataResult> updateLastActive({String? provider}) async{
    final accessToken = await getAccessToken(provider: provider);

    if(accessToken == null){
      return DataResult(success: false, message: "User is not signed in yet for active logging", code: "");
    }

    final userInfo = NUIUserInfo(userId: int.parse(accessToken.getUserId()));
    final result = await _api.updateUserInfo(info: userInfo);
    return DataResult(success: result.success, message: result.message, code: result.code.toString());
  }

}

class NUIAuthBuilder{
  late String _baseUrl;
  late String _appVersion;
  HeadersBuilder? _headers;
  NUIWebBuilder? _webBuilder;
  bool? _saveTokenWithSecureStorage;
  String? _provider;

  NUIAuthBuilder baseUrl(String baseUrl){
    _baseUrl = baseUrl;
    return this;
  }

  NUIAuthBuilder appVersion(String appVersion){
    _appVersion = appVersion;
    return this;
  }

  ///The header of NUIWeb will be replaced with the default headers from NUIAuth.
  NUIAuthBuilder webBuilder(NUIWebBuilder webBuilder){
    _webBuilder = webBuilder;
    return this;
  }

  NUIAuthBuilder saveTokenWithSecureStorage(bool secureStorage){
    _saveTokenWithSecureStorage = secureStorage;
    return this;
  }

  NUIAuthBuilder provider(String provider){
    _provider = provider;
    return this;
  }

  NUIAuthBuilder headers(HeadersBuilder headers){
    _headers = headers;
    return this;
  }

  NUIAuth build(){
    NUISharedPrefBuilder().build();
    return NUIAuth._builder(this);
  }
}

class NUIAuthWebInterceptor extends NUIWebInterceptor{
  NUIAuthWebInterceptor({this.onTokenExpired, this.provider});
  Function? onTokenExpired;
  String? provider;

  @override
  Future<NUIWebInterceptorResponse> intercept(String url, NUIWebMethod method, dynamic, Map<String, String>? headers, dynamic body) async{
    final accessToken = await NUIAuth.get().getAccessToken(provider: provider);
    if (accessToken?.isExpired(leeway: 0) == true && !contains(url, "auth/renewAccessToken") && !contains(url, "auth/login") && !contains(url, "auth/signup") && !contains(url, "auth/forgot-password") && !contains(url, "auth/sso-login") && !contains(url, "auth/logout") && !contains(url, "app-versions")) {
      logNUI(NUIAuth._MODULE, "Access token expired, proceed to refresh token");
//      final response = await NUIAuth.get()._api.refreshToken(request: NUIAuthLoginReq(grantType: NUIAuth.GRANT_TYPE_REFRESH_TOKEN, refreshToken: accessToken.refreshToken));
      final response = await NUIAuth.get().renewToken(provider: provider);

      if (response.success == true && response.data != null) {
        // save the new access token and refresh token
        final body = response.data;
        logNUI(NUIAuth._MODULE, "Token refreshed, new access token : ${body?.accessToken}, refresh token : ${body?.refreshToken}");
        return NUIWebInterceptorResponse(success: true, message: NUIAuthMessages.TOKEN_REFRESHED, interceptResponse: false);
      }
      else{
        logNUI(NUIAuth._MODULE, "Failed to refresh expired token");
        if(onTokenExpired != null) onTokenExpired!();
        return NUIWebInterceptorResponse(success: false, message: NUIAuthMessages.FAILED_TO_REFRESH_TOKEN_LOGIN, interceptResponse: true);
      }
    }
    else{
      return NUIWebInterceptorResponse(success: true, message: NUIAuthMessages.TOKEN_VALID, interceptResponse: false);
    }
  }

}