import 'dart:io';
import 'package:nui_auth/domain/nui_user_activity.dart';
import 'package:nui_auth/web/json/data_result_factory.dart';
import 'package:nui_auth/web/json/resp/nui_auth_standard_resp.dart';
import 'package:nui_auth/web/nui_auth_web_json.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/model/interfaces.dart';
import 'package:nui_web/nui_web.dart';
import '../domain/nui_user_info.dart';
import 'json/req/nui_auth_change_password.dart';
import 'json/req/nui_auth_forgot_password.dart';
import 'json/req/nui_auth_login.dart';
import 'json/req/nui_auth_renew_token.dart';
import 'json/req/nui_auth_signup.dart';

abstract class _Apis{
  Future<DataResult<bool>> signup({required NUIAuthSignUpReq request}); //Sign up
  Future<DataResult<NUIAuthLoginResp>> login({required NUIAuthLoginReq request}); //Login
  Future<DataResult<NUIAuthLoginResp>> renewToken({required NUIAuthRenewTokenReq request}); //Renew access token and refresh token
  Future<DataResult<bool>> forgotPassword({required NUIAuthForgotPasswordReq request}); //Forgot password
  Future<DataResult<bool>> changePassword({required NUIAuthChangePasswordReq request}); //Change password
  Future<DataResult<bool>> userProfile({String? provider});
  Future<DataResult<bool>> logout({required String accessToken}); //Logout
  Future<DataResult<NUIAuthLoginResp>> adminLogin({required NUIAuthLoginReq request}); //Admin / System login

  //TODO: Yet to implement, currently iAuth v3 doesn't have api below
  Future<DataResult<NUIAuthLoginResp>> ssoLogin({required NUIAuthSSOSignUpReq request}); //Login with SSO (Single Sign On)
  Future<NUIStandardHttpResponse> verifyEmail({required NUIAuthVerifyEmailReq request}); //Verify email after registration
  Future<NUIStandardHttpResponse> resendEmailVerificationCode({required NUIAuthVerifyEmailReq request}); //Resend email verification code
  Future<NUIStandardHttpResponse> checkTokenValidity(); //Check token's validity
  Future<NUIStandardHttpResponse> updateUserStatusOnline(); //Update status to online
  Future<NUIStandardHttpResponse> updateUserStatusOffline(); //Update status to offline
  Future<NUIStandardHttpResponse> updateUserStatusIdle(); //Update status to idle
  Future<NUIStandardHttpResponse> logUserActivity({required NUIUserActivity activity}); //Save user log activity
  Future<NUIStandardHttpResponse> updateUserInfo({required NUIUserInfo info}); //Save user information
}

class NUIAuthApi implements _Apis{
  static const String MODULE = "NUIAuthApiModule";

  NUIWeb _getWeb() => NUIWeb.get(module: MODULE);

  DataResultFactory _dataResultFactory = DataResultFactory();

  void _processResponseMessage(NUIHttpResponse response){
    if(response.body?.data?.containsKey("message") == true){
      response.body?.message = response.body.data["message"];
    }
    else if(response.code != HttpStatus.ok){
      final maskedResponse = NUIStandardHttpResponse(success: false, message: '');

      try {
        if (response.responseBody != null && response.responseBody is Map) {
          maskedResponse.message = (response.responseBody as Map)["message"];
        }
      }catch(e){
        logNUI("NUIAuthApi", "Failed to get error message from response");
      }

      response.body = maskedResponse;
      //API not successful
    }
  }

  @override
  Future<DataResult<bool>> signup({required NUIAuthSignUpReq request}) async{
    final resp = await _getWeb().doPOST(
      url: "user/registration", 
      bodyEnt: request,
      entity: NUIAuthStandardRespEnt(NUIEntType.MAP, (value) => asMap(value))
    );
    return _dataResultFactory.defineDataResult<bool>(resp, successDataIfNull: true, failureData: false);
    // final result = await _getWeb().doPOST(url: "auth/signup", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => signup Failed");
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> login({required NUIAuthLoginReq request}) async{
    final resp = await _getWeb().doPOST(
        url: "auth/authenticate",
        headers: {"Authorization": ""},
        bodyEnt: request,
        entity: NUIAuthStandardRespEnt(NUIEntType.OBJECT, (value) => asObject(NUIAuthLoginRespEnt(), value))
    );
    return _dataResultFactory.defineDataResult<NUIAuthLoginResp>(resp);
    // final result = await _getWeb().doPOST(url: "auth/authenticate", headers: {"Authorization": ""}, bodyEnt: request, entity: NUIAuthLoginRespEnt());
    // if(result.responseBody != null){
    //   final rawResponse = result.responseBody;
    //   if(rawResponse is Map){
    //     final respMessage = rawResponse["message"];
    //
    //     if(respMessage != null && respMessage is String){
    //       result.message = respMessage;
    //     }
    //   }
    // }
    // return DataResult<NUIAuthLoginResp>(success: result.success, message: result.message, data: result.body, code: result.code.toString());
  }


  @override
  Future<DataResult<NUIAuthLoginResp>> renewToken({required NUIAuthRenewTokenReq request}) async{
    final resp = await _getWeb().doPOST(
        url: "auth/token/renew",
        bodyEnt: request,
        entity: NUIAuthStandardRespEnt(NUIEntType.OBJECT, (value) => asObject(NUIAuthLoginRespEnt(), value))
    );
    return _dataResultFactory.defineDataResult<NUIAuthLoginResp>(resp);
  }

  @override
  Future<DataResult<bool>> forgotPassword({required NUIAuthForgotPasswordReq request}) async{
    final resp = await _getWeb().doPOST(
        url: "user/forgot-password",
        bodyEnt: request,
        entity: NUIAuthStandardRespEnt(NUIEntType.MAP, (value) => asMap(value))
    );
    return _dataResultFactory.defineDataResult<bool>(resp, successDataIfNull: true, failureData: false);
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => forgotPassword Failed");
  }

  @override
  Future<DataResult<bool>> changePassword({required NUIAuthChangePasswordReq request}) async{
    final resp = await _getWeb().doPOST(
        url: "user/reset-password",
        bodyEnt: request,
        entity: NUIAuthStandardRespEnt(NUIEntType.MAP, (value) => asMap(value))
    );
    return _dataResultFactory.defineDataResult<bool>(resp, successDataIfNull: true, failureData: false);
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => changePassword Failed");
  }

  @override
  Future<DataResult<bool>> userProfile({String? provider}) async {
    final resp = await _getWeb().doGET(
        url: "user/profile",
        entity: NUIAuthStandardRespEnt(NUIEntType.MAP, (value) => asMap(value))
    );
    return _dataResultFactory.defineDataResult(resp);

  }

  @override
  Future<DataResult<bool>> logout({required String accessToken}) async{
    final resp = await _getWeb().doPOST(
        url: "auth/logout",
        headers: {"Authorization": accessToken },
        entity: NUIAuthStandardRespEnt(NUIEntType.MAP, (value) => asMap(value))
    );
    return _dataResultFactory.defineDataResult<bool>(resp, successDataIfNull: true, failureData: false);
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => logout Failed");
  }


  @override
  Future<DataResult<NUIAuthLoginResp>> adminLogin({required NUIAuthLoginReq request}) async{
    // final result = await _getWeb().doPOST(url: "auth/admin/login", bodyEnt: request, entity: NUIAuthLoginRespEnt());
    // if(result.responseBody != null){
    //   final rawResponse = result.responseBody;
    //   if(rawResponse is Map){
    //     final respMessage = rawResponse["message"];
    //
    //     if(respMessage != null && respMessage is String){
    //       result.message = respMessage;
    //     }
    //   }
    // }
    // return DataResult<NUIAuthLoginResp>(success: result.success, message: result.message, data: result.body, code: result.code.toString());
    // TODO: implement adminLogin
    throw UnimplementedError();
  }

  @override
  Future<NUIStandardHttpResponse> checkTokenValidity() async{
    // final result = await _getWeb().doPOST(url: "auth/token/auth", entity: NUIStandardHttpResponseEntity());
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => checkTokenValidity Failed");
    // TODO: implement checkTokenValidity
    throw UnimplementedError();
  }

  @override
  Future<NUIStandardHttpResponse> logUserActivity({required NUIUserActivity activity}) async{
    // final result = await _getWeb().doPOST(url: "user-activities", bodyEnt: activity, entity: NUIStandardHttpResponseEntity());
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => logUserActivity Failed");
    // TODO: implement logUserActivity
    throw UnimplementedError();
  }

  @override
  Future<NUIStandardHttpResponse> updateUserInfo({required NUIUserInfo info}) async{
    // final result = await _getWeb().doPATCH(url: "user-info", bodyEnt: info, entity: NUIStandardHttpResponseEntity());
    // _processResponseMessage(result);
    // return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => updateUserInfo Failed");
    // TODO: implement updateUserInfo
    throw UnimplementedError();
  }



  @override
  Future<NUIStandardHttpResponse> resendEmailVerificationCode({required NUIAuthVerifyEmailReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/signup/verify/resend", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result);
    return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => resendEmailVerificationCode Failed");
  }



  @override
  Future<DataResult<NUIAuthLoginResp>> ssoLogin({required NUIAuthSSOSignUpReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/sso-login", bodyEnt: request, entity: NUIAuthLoginRespEnt());
    return DataResult<NUIAuthLoginResp>(success: result.success, message: result.message, data: result.body, code: result.code.toString());
  }

  @override
  Future<NUIStandardHttpResponse> updateUserStatusIdle() async{
    final result = await _getWeb().doPUT(url: "users/activity/idle", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result);
    return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => updateUserStatusIdle Failed");
  }

  @override
  Future<NUIStandardHttpResponse> updateUserStatusOffline() async{
    final result = await _getWeb().doPUT(url: "users/activity/offline", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result);
    return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => updateUserStatusOffline Failed");
  }

  @override
  Future<NUIStandardHttpResponse> updateUserStatusOnline() async{
    final result = await _getWeb().doPUT(url: "users/activity/online", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result);
    return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => updateUserStatusOnline Failed");
  }

  @override
  Future<NUIStandardHttpResponse> verifyEmail({required NUIAuthVerifyEmailReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/signup/verify", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result);
    return result.body ?? NUIStandardHttpResponse(success: false, message: "NUIAuthApi => verifyEmail Failed");
  }

}